Index: src/Fl_Shared_Image.cxx
===================================================================
--- src/Fl_Shared_Image.cxx	(revision 12472)
+++ src/Fl_Shared_Image.cxx	(working copy)
@@ -3,7 +3,7 @@
 //
 // Shared image code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-2016 by Bill Spitzak and others.
+// Copyright 1998-2017 by Bill Spitzak and others.
 //
 // This library is free software. Distribution and use rights are outlined in
 // the file "COPYING" which should have been included with this file.  If this
@@ -100,13 +100,27 @@
 int
 Fl_Shared_Image::compare(Fl_Shared_Image **i0,		// I - First image
                          Fl_Shared_Image **i1) {	// I - Second image
-  int i = strcmp((*i0)->name(), (*i1)->name());
 
-  if (i) return i;
-  else if (((*i0)->w() == 0 && (*i1)->original_) ||
-           ((*i1)->w() == 0 && (*i0)->original_)) return 0;
-  else if ((*i0)->w() != (*i1)->w()) return (*i0)->w() - (*i1)->w();
-  else return (*i0)->h() - (*i1)->h();
+  // compare name
+  int diff = strcmp((*i0)->name(), (*i1)->name());
+  if (diff) return diff;
+
+  // test for exact match of width and height
+  if (((*i0)->w() == (*i1)->w()) && ((*i0)->h() == (*i1)->h()))
+    return 0;
+
+  // image with zero width matches original image
+  if (((*i0)->w() == 0) && ((*i1)->original_)) return 0;
+  if (((*i1)->w() == 0) && ((*i0)->original_)) return 0;
+
+  // sort original image before others of same name to ensure that the
+  // binary search with width == 0 finds the original image (STR #3410)
+  if ((*i0)->original_) return -1;
+  if ((*i1)->original_) return 1;
+
+  // sort remaining cases by width and then by height
+  diff = (*i0)->w() - (*i1)->w();
+  return diff ? diff : (*i0)->h() - (*i1)->h();
 }
 
 
